@@ -1,19 +1,10 @@
 // Copyright (C) 2019 Sebastian Dröge <sebastian@centricular.com>
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 51 Franklin Street, Suite 500,
-// Boston, MA 02110-1335, USA.
+// SPDX-License-Identifier: MPL-2.0
 
 use gio::prelude::*;
 
@@ -41,9 +32,7 @@ fn create_pipeline() -> (gst::Pipeline, gst::Pad, gst::Element, gtk::Widget) {
         .upcast();
 
     let fallbackswitch = gst::ElementFactory::make("fallbackswitch", None).unwrap();
-    fallbackswitch
-        .set_property("timeout", &gst::ClockTime::SECOND)
-        .unwrap();
+    fallbackswitch.set_property("timeout", gst::ClockTime::SECOND);
 
     let decodebin = gst::ElementFactory::make("decodebin", None).unwrap();
     let videoconvert = gst::ElementFactory::make("videoconvert", None).unwrap();
@@ -63,15 +52,15 @@ fn create_pipeline() -> (gst::Pipeline, gst::Pad, gst::Element, gtk::Widget) {
     let (video_sink, video_widget) =
         //if let Some(gtkglsink) = gst::ElementFactory::make("gtkglsink", None) {
         //    let glsinkbin = gst::ElementFactory::make("glsinkbin", None).unwrap();
-        //    glsinkbin.set_property("sink", &gtkglsink).unwrap();
+        //    glsinkbin.set_property("sink", &gtkglsink);
 
-        //    let widget = gtkglsink.get_property("widget").unwrap();
-        //    (glsinkbin, widget.get::<gtk::Widget>().unwrap().unwrap())
+        //    let widget = gtkglsink.property::<gtk::Widget>("widget");
+        //    (glsinkbin, widget)
         //} else
         {
             let sink = gst::ElementFactory::make("gtksink", None).unwrap();
-            let widget = sink.property("widget").unwrap();
-            (sink, widget.get::<gtk::Widget>().unwrap())
+            let widget = sink.property::<gtk::Widget>("widget");
+            (sink, widget)
         };
 
     pipeline
@@ -210,7 +199,7 @@ fn create_ui(app: &gtk::Application) {
         bus.remove_watch().unwrap();
 
         if let Some(timeout_id) = timeout_id.borrow_mut().take() {
-            glib::source_remove(timeout_id);
+            timeout_id.remove();
         }
     });
 }
