@@ -1,51 +1,29 @@
 // Copyright (C) 2019 Sebastian Dröge <sebastian@centricular.com>
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 51 Franklin Street, Suite 500,
-// Boston, MA 02110-1335, USA.
+// SPDX-License-Identifier: MPL-2.0
 
 use gst::glib;
 
-#[cfg(not(feature = "v1_18"))]
-use self::gst_base::prelude::*;
-#[cfg(not(feature = "v1_18"))]
-use self::gst_base::subclass::prelude::*;
-#[cfg(not(feature = "v1_18"))]
-use crate::gst_base_compat as gst_base;
-
-#[cfg(feature = "v1_18")]
 use gst_base::prelude::*;
-#[cfg(feature = "v1_18")]
 use gst_base::subclass::prelude::*;
 
-#[cfg(not(feature = "v1_18"))]
-use glib::subclass::prelude::*;
-#[cfg(not(feature = "v1_18"))]
-use gst::subclass::prelude::*;
 use gst::{gst_debug, gst_error, gst_info, gst_log, gst_warning};
 
 use once_cell::sync::Lazy;
 
 use std::sync::{Mutex, RwLock};
 
-#[derive(Debug, Eq, PartialEq, Ord, PartialOrd, Hash, Clone, Copy, glib::GEnum)]
+#[derive(Debug, Eq, PartialEq, Ord, PartialOrd, Hash, Clone, Copy, glib::Enum)]
 #[repr(u32)]
-#[genum(type_name = "GstFallbackSwitchStreamHealth")]
+#[enum_type(name = "GstFallbackSwitchStreamHealth")]
 pub(crate) enum StreamHealth {
-    #[genum(name = "Data flow is inactive or late", nick = "inactive")]
+    #[enum_value(name = "Data flow is inactive or late", nick = "inactive")]
     Inactive = 0,
-    #[genum(name = "Data is currently flowing in the stream", nick = "present")]
+    #[enum_value(name = "Data is currently flowing in the stream", nick = "present")]
     Present = 1,
 }
 
@@ -128,6 +106,7 @@ impl Default for Settings {
 }
 
 impl OutputState {
+    #[allow(clippy::blocks_in_if_conditions)]
     fn health(
         &self,
         settings: &Settings,
@@ -212,8 +191,8 @@ impl FallbackSwitch {
 
             if pts.is_none()
                 || new_running_time
-                    .zip(target_running_time.into())
-                    .map_or(false, |(new, target)| new <= target)
+                    .opt_le(target_running_time.into())
+                    .unwrap_or(false)
             {
                 gst_debug!(CAT, obj: pad, "Dropping trailing buffer {:?}", buffer);
                 pad.drop_buffer();
@@ -281,16 +260,13 @@ impl FallbackSwitch {
         }
 
         let cur_running_time = cur_running_time.into();
-        let (is_late, deadline) = cur_running_time
-            .zip(agg.latency())
-            .zip(running_time)
-            .map_or(
-                (false, None),
-                |((cur_running_time, latency), running_time)| {
-                    let dealine = running_time + latency + 40 * gst::ClockTime::MSECOND;
-                    (cur_running_time > dealine, Some(dealine))
-                },
-            );
+        let (is_late, deadline) = match (cur_running_time, agg.latency(), running_time) {
+            (Some(cur_running_time), Some(latency), Some(running_time)) => {
+                let deadline = running_time + latency + 40 * gst::ClockTime::MSECOND;
+                (cur_running_time > deadline, Some(deadline))
+            }
+            _ => (false, None),
+        };
 
         if is_late {
             gst_debug!(
@@ -301,12 +277,12 @@ impl FallbackSwitch {
                 deadline.display(),
             );
 
-            if state.last_output_time.zip(running_time).map_or(
-                false,
-                |(last_output_time, running_time)| {
-                    last_output_time + settings.timeout <= running_time
-                },
-            ) {
+            let is_late = state
+                .last_output_time
+                .opt_add(settings.timeout)
+                .opt_le(running_time);
+
+            if let Some(true) = is_late {
                 /* This buffer arrived too late - we either already switched
                  * to the other pad or there's no point outputting this anyway */
                 gst_debug!(
@@ -429,13 +405,14 @@ impl FallbackSwitch {
             };
 
             if !ignore_timeout {
+                let timed_out = state
+                    .last_output_time
+                    .opt_add(settings.timeout)
+                    .opt_le(running_time)
+                    .unwrap_or(true);
+
                 // Get the next one if this one is before the timeout
-                if state.last_output_time.zip(running_time).map_or(
-                    false,
-                    |(last_output_time, running_time)| {
-                        last_output_time + settings.timeout > running_time
-                    },
-                ) {
+                if !timed_out {
                     gst_debug!(
                         CAT,
                         obj: backup_pad,
@@ -553,10 +530,7 @@ impl FallbackSwitch {
         let base_time = agg.base_time();
 
         let cur_running_time = if let Some(clock) = clock {
-            clock
-                .time()
-                .zip(base_time)
-                .and_then(|(time, base_time)| time.checked_sub(base_time))
+            clock.time().opt_checked_sub(base_time).ok().flatten()
         } else {
             gst::ClockTime::NONE
         };
@@ -699,7 +673,7 @@ impl ObjectImpl for FallbackSwitch {
     fn properties() -> &'static [glib::ParamSpec] {
         static PROPERTIES: Lazy<Vec<glib::ParamSpec>> = Lazy::new(|| {
             vec![
-                glib::ParamSpec::new_uint64(
+                glib::ParamSpecUInt64::new(
                     "timeout",
                     "Timeout",
                     "Timeout in nanoseconds",
@@ -708,21 +682,21 @@ impl ObjectImpl for FallbackSwitch {
                     DEFAULT_TIMEOUT.nseconds() as u64,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_object(
+                glib::ParamSpecObject::new(
                     "active-pad",
                     "Active Pad",
                     "Currently active pad. Writes are ignored if auto-switch=true",
                     gst::Pad::static_type(),
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_PLAYING,
                 ),
-                glib::ParamSpec::new_boolean(
+                glib::ParamSpecBoolean::new(
                     "auto-switch",
                     "Automatically switch pads",
                     "Automatically switch pads (If true, prefer primary sink, otherwise manual selection via the active-pad property)",
                     DEFAULT_AUTO_SWITCH,
                     glib::ParamFlags::READWRITE| gst::PARAM_FLAG_MUTABLE_READY,
                 ),
-                glib::ParamSpec::new_enum(
+                glib::ParamSpecEnum::new(
                     "primary-health",
                     "Primary stream state",
                     "Reports the health of the primary stream on the sink pad",
@@ -730,7 +704,7 @@ impl ObjectImpl for FallbackSwitch {
                     DEFAULT_STREAM_HEALTH as i32,
                     glib::ParamFlags::READABLE,
                 ),
-                glib::ParamSpec::new_enum(
+                glib::ParamSpecEnum::new(
                     "fallback-health",
                     "Fallback stream state",
                     "Reports the health of the fallback stream on the fallback_sink pad",
@@ -738,11 +712,11 @@ impl ObjectImpl for FallbackSwitch {
                     DEFAULT_STREAM_HEALTH as i32,
                     glib::ParamFlags::READABLE,
                 ),
-                glib::ParamSpec::new_boolean(
+                glib::ParamSpecBoolean::new(
                     "immediate-fallback",
                     "Immediate fallback",
                     "Forward the fallback stream immediately at startup, when the primary stream is slow to start up and immediate output is required",
-                    DEFAULT_AUTO_SWITCH,
+                    DEFAULT_IMMEDIATE_FALLBACK,
                     glib::ParamFlags::READWRITE| gst::PARAM_FLAG_MUTABLE_READY,
                 ),
             ]
@@ -842,6 +816,8 @@ impl ObjectImpl for FallbackSwitch {
     }
 }
 
+impl GstObjectImpl for FallbackSwitch {}
+
 impl ElementImpl for FallbackSwitch {
     fn metadata() -> Option<&'static gst::subclass::ElementMetadata> {
         static ELEMENT_METADATA: Lazy<gst::subclass::ElementMetadata> = Lazy::new(|| {
@@ -1149,10 +1125,10 @@ impl AggregatorImpl for FallbackSwitch {
                 audio_info.rate() as u64 * audio_info.bpf() as u64,
             )
         } else if let Some(ref video_info) = pad_state.video_info {
-            if *video_info.fps().numer() > 0 {
+            if video_info.fps().numer() > 0 {
                 gst::ClockTime::SECOND.mul_div_floor(
-                    *video_info.fps().denom() as u64,
-                    *video_info.fps().numer() as u64,
+                    video_info.fps().denom() as u64,
+                    video_info.fps().numer() as u64,
                 )
             } else {
                 gst::ClockTime::NONE
@@ -1177,16 +1153,13 @@ impl AggregatorImpl for FallbackSwitch {
                 audio_info.bpf(),
             )
         } else if pad_state.video_info.is_some() {
-            let stop = pts.zip(duration).map(|(pts, duration)| pts + duration);
+            let stop = pts.opt_add(duration);
             segment.clip(pts, stop).map(|(start, stop)| {
                 {
                     let buffer = buffer.make_mut();
                     buffer.set_pts(start);
                     if duration.is_some() {
-                        buffer.set_duration(
-                            stop.zip(start)
-                                .and_then(|(stop, start)| stop.checked_sub(start)),
-                        );
+                        buffer.set_duration(stop.opt_checked_sub(start).ok().flatten());
                     }
                 }
 
